-- some fancy schmancy stuff to make us look cool. 
local LOGO = [[
   _____                            _             __      __   _          
  / ____|                          | |            \ \    / /  (_)         
 | |     ___  _ __ ___  _ __  _   _| |_ ___ _ __   \ \  / /__  _  ___ ___ 
 | |    / _ \| '_ ` _ \| '_ \| | | | __/ _ \ '__|   \ \/ / _ \| |/ __/ _ \
 | |___| (_) | | | | | | |_) | |_| | ||  __/ |       \  / (_) | | (_|  __/
  \_____\___/|_| |_| |_| .__/ \__,_|\__\___|_|        \/ \___/|_|\___\___|
                       | |
@Remen for more info   |_| ]]

local ABSTRACT = [[
Computerized voice for user settable health percentages (audible alerting) and optional HUD message.
]]

local PROPS = [[
	* Thanks to Raybondo for assistnace with the gksound and TextToSpeech functionality!
	* Original idea by Zenko_Kaz09 @ https://www.vendetta-online.com/x/msgboard/3/36395
	* Thanks to Mr. Whale and Scion Spy and A-Dawg (who deshielded my goli!) for testing the damage alerts
	* Thanks to John Waters for asking politely for a mobile version (LITE) that had a smaller download footprint
]]

local LITE = true -- set to true for LITE distribution

local PLUGIN_NAME = "Computer Voice" -- this is used for display.
local PLUGIN_COMMAND = "cv"
local PLUGIN_COMMAND_SLASH = "/cv"

local USAGE = [[| 
| Usage:
|  See current settings: /cv show
|  Set armor or shield announcements by percentage: /cv [armor|shield] <#>
|    (To stop announcing, reissue command)
|  Toggle HUD alerting: /cv hud [on|off]
|  Toggle voice alerting: /cv voice [on|off]
|  Toggle mobile mode (text-to-speech): /cv voice [on|off]
|  Toggle debug mode: /cv debug [on|off]
|]]



declare('CompVoice', CompVoice or {})

-- ----------
-- BEGIN plugin data container

CompVoice = {
	-- define most of our data structures, values, and types
		version = "2.0.1", -- major ver . feature update . bug fix/minor changes
		--[[ 
			Version history:
			1.0.0 - Prototype; MVP functionality - Initial release
			1.1.0 - Integrated LITE functionality for size limited distribution
			2.0.0 - Integrated MOBILE functionality; uses text-to-speech, fixed sound queuing and pacing
			2.0.1 - replaced missing sound files
		]]--
	
	-- save file data
		systemnotes_config_save_file = 44400000444, -- used to save config data (any arbitrarily large number; 11 digits is good)
		
	-- control vars
		loggedin = false, -- is the player logged in?
		initialized = false, -- has the plugin been been initialized? (used for reloads)
		ship_has_shields = false, -- does this ship normally have shields?
		last_announced_armor_pct = 100, -- what was the last announced percentage
		last_announced_shields_pct = 100, -- what was the last announced percentage
		last_armor_pct = 100, -- what was our actual last armor level
		last_shields_pct = 100, -- what was our actual last shields level
		shields_offline = false, -- are our shields offline?
		ship_docked = false, -- we don't know yet...
		
	-- sound management variables
		sound_looping = false,
		sound_queue = {},
		sound_playing = "",
		sound_objects = {},

	-- set up our default configuration data
		DefaultConfig = {
			debug_mode = false, -- are we debugging?
			-- add any default config data vars/tables below here
			armor_pcts = {
				[5] = true,
				[25] = true,
				[50] = true,
				[75] = true,
			}, -- shield_pcts
			shield_pcts = {
				[5] = true,
				[25] = true,
				[50] = true,
				[75] = true,
			}, -- shield_pcts
			hud_alerts = false,
			voice_alerts = true,
			mobile_mode = false,
			volume = 1,
		}, -- DefaultConfig

	-- active configuration data, the default is merged into this container the first time this plugin runs
	-- after that first run, the config comes from its save file
		Config = {
		},
}

-- EVENT function declarations
	CV_PlayerLoggedIn = {} -- function that gets called when we login
	CV_ShipGotRepped = {} -- called when we repair/dock/etc.
	CV_ShipGotDamaged = {} -- called when we get shot

-- END plugin data container
-- ----------
-- BEGIN save/load data functions

	function CompVoice.SaveData()
		CompVoice.Config["version"] = CompVoice.version -- seed the plugin version into the config save file
		SaveSystemNotes(spickle(CompVoice.Config), CompVoice.systemnotes_config_save_file)
	end

-- -----

	function CompVoice.LoadData()
		CompVoice.Config = unspickle(LoadSystemNotes(CompVoice.systemnotes_config_save_file))
		--  if first run, populate initial configuration with defaults
		if not CompVoice.Config["version"] then 
			CompVoice.Config = CompVoice.DefaultConfig
			if not CompVoice.Config.pcts then CompVoice.Config.pcts = {} end
		end 
	end

-- END save/load data functions
-- ----------
-- BEGIN pretty print functions

	-- pretty print a header
	local function CV_Visual_BoxHeader(text)
		print(string.format("\127FFFFFF+-  %-20s   -----------------------------------------------------------------------------------------------------\127o", PLUGIN_NAME))
		if (text) then 
			print("\127FFFFFF| \127A0A0A0 " .. text .. "\127o")
			print("\127FFFFFF+---------------------------------------------------------------------------------------------------------------------------------\127o")
		end
	end

-- -----

	-- pretty print a text line
	local function CV_Visual_BoxText(text)
		print ("\127FFFFFF| \127A0A0A0" .. text .. "\127o")
	end

-- -----

	-- pretty print a display line
	local function CV_Visual_BoxLine()
		print("\127FFFFFF+---------------------------------------------------------------------------------------------------------------------------------\127o")
	end

-- -----	

	-- pretty print a footer
	local function CV_Visual_BoxFooter(text)
		print("\127FFFFFF+---------------------------------------------------------------------------------------------------------------------------------\127o")
		if (text) then 
			print("\127FFFFFF| \127A0A0A0 " .. text .. "\127o")
			print("\127FFFFFF+---------------------------------------------------------------------------------------------------------------------------------\127o")
		end
	end

-- -----	
	
	-- print the word "on" or "off" based on the passed boolean value
	local function CV_Visual_ToggleStatus(tf)
		if tf then
			return "\12700FF00on\127o" 
		else
			return "\127FF0000off\127o"
		end
	end

	-- print the word "on" or "off" based on the passed boolean value
	local function CV_Visual_TF(tf)
		if tf then
			return "\12700FF00true\127o" 
		else
			return "\127FF0000false\127o"
		end
	end

-- ----------
-- BEGIN COMM functions
	
	local function CV_SendToDebug(message)
		if CompVoice.Config.debug_mode then console_print("\127FFA0A0" .. PLUGIN_NAME .. " DEBUG:\127A0A0A0 " .. message .. "\127o") end
	end

-- END COMM functions
-- ----------
-- BEGIN SOUND management
	
	local function CV_QueueSoundPop() -- FIFO
		if CompVoice.sound_queue[1] then
			local sound = CompVoice.sound_queue[1]
			table.remove(CompVoice.sound_queue, 1)
			-- do we need to renumber these?
			CV_SendToDebug("popping: " .. sound)
			return sound
		else
			return false
		end
	end

	local function CV_QueueSoundClear()
		for k,v in pairs(CompVoice.sound_queue) do
			CompVoice.sound_queue[k] = nil
		end
	end

	local function CV_QueueSoundPlay(what)
		if what then CV_SendToDebug("playing: " .. what); CompVoice.sound_objects[what]:Play(); CompVoice.sound_playing = what; end
		CV_SendToDebug(CV_Visual_TF(CompVoice.sound_objects[CompVoice.sound_playing]:IsPlaying()))
		if CompVoice.sound_objects[CompVoice.sound_playing]:IsPlaying() then
			CV_SendToDebug("sound playing... looping.")
			if CompVoice.sound_looping then Timer ():SetTimeout (100, function () CV_QueueSoundPlay(); end) end
		else
			-- we're not playing a sound, so grab the next and play it, if there isn't one then stop looping
			local soundfile = CV_QueueSoundPop() -- pop a sound file off the stack!
			if soundfile then
				CV_SendToDebug("got: " .. soundfile)
				CV_QueueSoundPlay(soundfile)
			else
				CV_SendToDebug("stopping sound loop")
				CompVoice.sound_looping = false -- no sound file in queue, stop looping
				CompVoice.sound_playing = "" -- we're not playing anything now
			end
		end
	end

	local function CV_QueueSound(what, reset) -- FIFO
		CV_SendToDebug("queuing: " .. what)
		if reset then CV_QueueSoundClear() end
		if not CompVoice.sound_looping then -- if we're not looping, then we can play a sound
			CompVoice.sound_looping = true
			CV_QueueSoundPlay(what) 
		else -- otherwise, let's push it onto the stack
			table.insert(CompVoice.sound_queue, what) -- append to the end
		end 
	end
	
-- END SOUND management functions
-- ----------
-- BEGIN plugin functions
	
	local function round(num)
		local under = math.floor(num)
		local upper = math.floor(num) + 1
		local underV = -(under - num)
		local upperV = upper - num
		if (upperV > underV) then
			return under
		else
			return upper
		end
	end
	
	local function CV_GetArmorShields()
		local armor, shields = GetPlayerHealth(GetCharacterID())
		if armor then armor = round(armor) else armor = 0 end
		if shields then
			shields = round(shields)
			CompVoice.ship_has_shields = true
		else
			shields = 0
			CompVoice.ship_has_shields = false
		end
		return armor, shields
	end

	local function CV_ShipChanged(...)
		local arg = {...}
		-- we've changed ships, let's reset our voice trackers
		local armor, shields = CV_GetArmorShields()
		CompVoice.last_announced_armor_pct = 100 -- reset voice
		CompVoice.last_announced_shields_pct = 100 -- reset voice
		CompVoice.last_armor_pct = armor
		CompVoice.last_shields_pct = shields
		if CompVoice.ship_has_shields and shields > 0 then 
			CompVoice.shields_offline = false
		else
			CompVoice.shields_offline = true
		end
	end

	-- called if the ship gets repaired/docks/etc.
	local function CV_ShipGotRepped(...)
		local arg = {...}
		local armor, shields = CV_GetArmorShields()
		CompVoice.last_armor_pct = armor
		CompVoice.last_shields_pct = shields

		if (armor > CompVoice.last_armor_pct) or (shields >  CompVoice.last_shields_pct) then

			-- we've been repaired, reset to 100%
			if armor > CompVoice.last_armor_pct then 
				CompVoice.last_announced_armor_pct = 100 -- reset voice
				CompVoice.last_armor_pct = armor
				if CompVoice.Config.voice_alerts then 
					if CompVoice.Config.mobile_mode then
						if TextToSpeech:IsSpeaking() then TextToSpeech:StopSpeaking() end
						TextToSpeech:Speak("ship repaired")
					else
						CV_QueueSound("ship_repaired", false)
					end
				end
			end
			
			-- we've been repaired, reset to 100%
			if CompVoice.ship_has_shields and shields > CompVoice.last_shields_pct then 
				CompVoice.last_announced_shields_pct = 100 -- reset voice
				CompVoice.last_shields_pct = shields
			end 

			-- Shields restored?
			if CompVoice.ship_has_shields and shields_offline and shields > 0 then 
				CompVoice.last_announced_shields_pct = 100 
				CompVoice.last_shields_pct = shields
				if CompVoice.Config.voice_alerts then 
					if CompVoice.Config.mobile_mode then
						if TextToSpeech:IsSpeaking() then TextToSpeech:StopSpeaking() end
						TextToSpeech:Speak("shields up")
					else
						CV_QueueSound("shields_up", false)
					end
				end
				CompVoice.shields_offline = false
			end 

		end
	end

	-- called if the ship gets shot/damaged
	local function CV_ShipGotDamaged(...)
		local arg = {...}
		local event = "UNKNOWN"
		local hit = ""
		local shooter = ""
		local objid = ""
		local damaged = false
		if arg[1] then event = arg[1] else event = "SHIP_UPDATED" end -- force something harmless if we don't have an event passed...
		
		if event == "PLAYER_GOT_HIT"  then
			if arg[2] then hit = arg[2] end
			if arg[3] then shooter = arg[3] end
			if arg[4] then objid = arg[4] end
			if arg[5] then damaged = arg[5] end
		else
			-- override passed values
			hit = GetCharacterID()
			damaged = false
		end
		
		CV_SendToDebug(event)
		
		local next_max_shields = 100
		local next_max_armor = 100
		local temp_keys = {}

		if GetPlayerName(hit) == CompVoice.loggedin then

			local armor, shields = CV_GetArmorShields()
			
			if armor == -1 and not CompVoice.ship_docked then 
				-- we've docked with a station or a capships
				if CompVoice.Config.mobile_mode then
					if TextToSpeech:IsSpeaking() then TextToSpeech:StopSpeaking() end
					TextToSpeech:Speak("docked")
				else
					CV_QueueSound("docked", true)
				end
				CompVoice.ship_docked = true
				return
			end

			if event == "SHIP_SPAWNED" then
				CompVoice.ship_docked = false
			end
			
			CompVoice.last_armor_pct = armor
			CompVoice.last_shields_pct = shields

			if ((armor < CompVoice.last_armor_pct) or (shields < CompVoice.last_shields_pct)) then damaged = true end
			if ((armor > CompVoice.last_armor_pct) or (shields > CompVoice.last_shields_pct)) and (damaged == false) then CV_ShipGotRepped(); return; end -- don't announce damage if we're being repped

			-- Voice audible alerting
			if CompVoice.Config.voice_alerts then

				if CompVoice.ship_has_shields and shields == 0 then 
					if CompVoice.Config.mobile_mode then
						if TextToSpeech:IsSpeaking() then TextToSpeech:StopSpeaking() end
						TextToSpeech:Speak("shields down")
					else
						CV_QueueSound("shields_down", false)
					end
					CompVoice.shields_offline = true
				else
					CompVoice.shields_offline = false 
				end

				temp_keys = {}
				for k,_ in pairs(CompVoice.Config.armor_pcts) do table.insert(temp_keys, k) end
				table.sort(temp_keys)

				for n,s in ipairs(temp_keys) do
					if s >= armor then next_max_armor = s; break end
				end
				if (next_max_armor ~= CompVoice.last_announced_armor_pct) then
					CV_SendToDebug("Armor at " .. armor .. " percent [ < " .. next_max_armor .. "]")
					if armor ~= -1 and next_max_armor ~= -1 then
						if CompVoice.Config.mobile_mode then
							if TextToSpeech:IsSpeaking() then TextToSpeech:StopSpeaking() end -- new speech supercedes old speech
							TextToSpeech:Speak("armor at " .. tostring(armor) .. " percent")
						else
							CV_QueueSound("armor_at", true) 
							if LITE then
								CV_QueueSound("f_"..tostring(next_max_armor), false) 
							else
								CV_QueueSound("f_"..tostring(armor), false) 
							end
							CV_QueueSound("percent", false) 
						end
					end
				end

				if CompVoice.ship_has_shields then
					temp_keys = {}
					for k,_ in pairs(CompVoice.Config.shield_pcts) do table.insert(temp_keys, k) end
					table.sort(temp_keys)

					for n,s in ipairs(temp_keys) do
						if s >= shields then next_max_shields = s; break end
					end
					if (next_max_shields ~= CompVoice.last_announced_shields_pct) then
						CV_SendToDebug("shields at " .. shields .. " percent [ < " .. next_max_shields .. "]")
						if shields ~= -1 and next_max_shields ~= -1 then
							if CompVoice.Config.mobile_mode then
								if TextToSpeech:IsSpeaking() then TextToSpeech:StopSpeaking() end -- new speech supercedes old speech
								TextToSpeech:Speak("shields at " .. tostring(shields) .. " percent")
							else
								CV_QueueSound("shields_at", true) 
								if LITE then
									CV_QueueSound("f_"..tostring(next_max_shields), false) 
								else
									CV_QueueSound("f_"..tostring(shields), false) 
								end
								CV_QueueSound("percent", false) 
							end
						end
					end
				end
			end

			-- HUD visual alerting
			if CompVoice.Config.hud_alerts then
				if (next_max_armor ~= CompVoice.last_announced_armor_pct) then
					if HUD.visible then HUD:PrintSecondaryMsg("Armor at " .. armor .. " percent [ < " .. next_max_armor .. "]") end
				end
				if CompVoice.ship_has_shields then
					if (next_max_shields ~= CompVoice.last_announced_shields_pct) then
						if HUD.visible then HUD:PrintSecondaryMsg("Shields at " .. shields .. " percent [ < " .. next_max_shields .. "]") end
					end
				end
			end

			CompVoice.last_announced_armor_pct = next_max_armor
			CompVoice.last_announced_shields_pct = next_max_shields

		end
	end
	
	local function CV_SetArmorPcts(pct)
		if pct > 0 and pct < 100 and not LITE or (LITE and ((pct==5) or (pct==25) or (pct==50) or (pct==75))) then 
			if CompVoice.Config.armor_pcts[tonumber(pct)] then 
				CompVoice.Config.armor_pcts[tonumber(pct)] = nil 
				CV_Visual_BoxText("Armor alerting for " .. pct .. " percent disabled.")			
			else 
				CompVoice.Config.armor_pcts[tonumber(pct)] = true
				CV_Visual_BoxText("Armor alerting at " .. pct .. " percent.")			
			end
		else
			if not LITE then
				CV_Visual_BoxText("Armor alerting can only be [ 1 .. 99 ] percent.")
			else
				CV_Visual_BoxText("Armor alerting can only be [ 5 | 25 | 50 | 75 ] percent.")			
			end
		end
	end

	local function CV_SetShieldPcts(pct)
		if pct > 0 and pct < 100 and not LITE or (LITE and ((pct==5) or (pct==25) or (pct==50) or (pct==75))) then 
			if CompVoice.Config.shield_pcts[tonumber(pct)] then 
				CompVoice.Config.shield_pcts[tonumber(pct)] = nil 
				CV_Visual_BoxText("Shield alerting for " .. pct .. " percent disabled.")			
			else 
				CompVoice.Config.shield_pcts[tonumber(pct)] = true
				CV_Visual_BoxText("Shield alerting at " .. pct .. " percent.")			
			end
		else
			if not LITE then
				CV_Visual_BoxText("Shield alerting can only be [ 1 .. 99 ] percent.")
			else
				CV_Visual_BoxText("Shield alerting can only be [ 5 | 25 | 50 | 75 ] percent.")			
			end
		end
	end
	
-- END plugin functions
-- ----------
-- BEGIN toggles

	-- return the status as text for display
	local function CV_ToggleStatus(toggle,name)
		if toggle then 
			return "\127FFFFFF" .. PLUGIN_NAME .. " " .. name .. " is \12700FF00enabled\127FFFFFF.\127o"
		else 
			return "\127FFFFFF" .. PLUGIN_NAME .. " " .. name .. " is \127FF0000disabled\127FFFFFF.\127o"
		end
	end

	-- set and display debug mode
	local function CV_DebugMode(tf)
		CompVoice.Config.debug_mode=tf
		CV_Visual_BoxText(CV_ToggleStatus(CompVoice.Config.debug_mode, "Debug Mode")) 
		CompVoice.SaveData()
	end

	-- set and display mobile text-to-speech mode
	local function CV_MobileMode(tf)
		CompVoice.Config.mobile_mode=tf
		CV_Visual_BoxText(CV_ToggleStatus(CompVoice.Config.mobile_mode, "Mobile text-to-speech")) 
		CompVoice.SaveData()
	end

	-- set and display HUD Alerting mode
	local function CV_HUDMode(tf)
		CompVoice.Config.hud_alerts=tf
		CV_Visual_BoxText(CV_ToggleStatus(CompVoice.Config.hud_alerts, "HUD Alerting")) 
		CompVoice.SaveData()
	end

	-- set and display voice alerting mode
	local function CV_VoiceMode(tf)
		CompVoice.Config.voice_alerts=tf
		CV_Visual_BoxText(CV_ToggleStatus(CompVoice.Config.voice_alerts, "Voice Alerting")) 
		CompVoice.SaveData()
	end

-- END toggles
-- ----------
-- BEGIN parser

	-- /COMMAND parser
	function CompVoice.SlashCommandParser(data, args)

		CV_Visual_BoxHeader("Welcome! This is " .. PLUGIN_NAME .. ", version " .. CompVoice.version)

		if args and args[1] then

			local valid_command = false
			local lc_command = string.lower(args[1])
			local lc_sub_command = ""
			local temp_keys = {}
			if args[2] then lc_sub_command = string.lower(args[2]) end

			if lc_command == "show" then
			
				temp_keys = {}
				for k,_ in pairs(CompVoice.Config.armor_pcts) do table.insert(temp_keys, k) end
				table.sort(temp_keys)
				local sline = ""
				for n,s in ipairs(temp_keys) do
					if sline ~= "" then sline = sline .. ", " end
					sline = sline .. s
				end
				CV_Visual_BoxText("Armor alerts: " .. sline)

				temp_keys = {}
				for k,_ in pairs(CompVoice.Config.shield_pcts) do table.insert(temp_keys, k) end
				table.sort(temp_keys)
				sline = ""
				for n,s in pairs(temp_keys) do
					if sline ~= "" then sline = sline .. ", " end
					sline = sline .. s
				end
				CV_GetArmorShields()
				if not CompVoice.ship_has_shields then sline = sline .. " [This ship has no shields]" end
				CV_Visual_BoxText("Shield alerts: " .. sline)
				
				CV_Visual_BoxText("Audible alerts: " .. CV_Visual_ToggleStatus(CompVoice.Config.voice_alerts))
				CV_Visual_BoxText("HUD alerts: " .. CV_Visual_ToggleStatus(CompVoice.Config.hud_alerts))
				CV_Visual_BoxText("Mobile mode: " .. CV_Visual_ToggleStatus(CompVoice.Config.mobile_mode))

				if CompVoice.Config.debug_mode then CV_Visual_BoxText("Debug Mode: " .. CV_Visual_ToggleStatus(CompVoice.Config.debug_mode)) end

				valid_command = true
			end

			if lc_command == "armor" then
				if (lc_sub_command~="") and (string.match(lc_sub_command,'(%d+)') == lc_sub_command) then
					CV_SetArmorPcts(tonumber(lc_sub_command))
				else
					CV_Visual_BoxText("Usage:")
					if LITE then
						CV_Visual_BoxText("-- '" .. PLUGIN_COMMAND_SLASH .. " armor [5|25|75]' to enable alerting for a given armor level.")
						CV_Visual_BoxText("-- for more levels, download the full version of this plugin!")
						CV_Visual_BoxText("-- '" .. PLUGIN_COMMAND_SLASH .. " armor <#>' to enable alerting for a given armor level.")
					end
				end
				valid_command = true
			end
			
			if lc_command == "shield" then
				if (lc_sub_command~="") and (string.match(lc_sub_command,'(%d+)') == lc_sub_command) then
					CV_SetShieldPcts(tonumber(lc_sub_command))
				else
					CV_Visual_BoxText("Usage:")
					if LITE then
						CV_Visual_BoxText("-- '" .. PLUGIN_COMMAND_SLASH .. " shield  [5|25|75]' to enable alerting for a given shield level.")
						CV_Visual_BoxText("-- for more levels, download the full version of this plugin!")
						CV_Visual_BoxText("-- '" .. PLUGIN_COMMAND_SLASH .. " shield <#>' to enable alerting for a given shield level.")
					end
				end
				valid_command = true
			end
			
			if lc_command == "voice" then 
				-- sub commands
				if lc_sub_command == "on" then 
					CV_VoiceMode(true); 
				elseif lc_sub_command == "off" then 
					CV_VoiceMode(false); 
				else
					-- no args passed, display help
					CV_Visual_BoxText("Usage:")
					CV_Visual_BoxText("-- '" .. PLUGIN_COMMAND_SLASH .. " voice on' to enable audible alerting.")
					CV_Visual_BoxText("-- '" .. PLUGIN_COMMAND_SLASH .. " voice off' to diable audible alerting.")
				end
				valid_command = true
			end

			-- HUD
			if lc_command == "hud" then 
				-- sub commands
				if lc_sub_command == "on" then 
					CV_HUDMode(true); 
				elseif lc_sub_command == "off" then 
					CV_HUDMode(false);
				else
					-- no args passed, display help
					CV_Visual_BoxText("Usage:")
					CV_Visual_BoxText("-- '" .. PLUGIN_COMMAND_SLASH .. " hud on' to enable HUD visual alerting.")
					CV_Visual_BoxText("-- '" .. PLUGIN_COMMAND_SLASH .. " hud off' to diable HUD visual alerting.")
				end
				valid_command = true
			end

			-- MOBILE
			if lc_command == "mobile" then 
				-- sub commands
				if lc_sub_command == "on" then 
					CV_MobileMode(true); 
				elseif lc_sub_command == "off" then 
					CV_MobileMode(false);
				else
					-- no args passed, display help
					CV_Visual_BoxText("Usage:")
					CV_Visual_BoxText("-- '" .. PLUGIN_COMMAND_SLASH .. " mobile on' to enable mobile voice-to-text visual alerting.")
					CV_Visual_BoxText("-- '" .. PLUGIN_COMMAND_SLASH .. " mobile off' to diable mobile voice-to-text alerting.")
				end
				valid_command = true
			end

			-- Debug 
			if lc_command == "debug" then 
				-- sub commands
				if lc_sub_command == "on" then 
					CV_DebugMode(true);
				elseif lc_sub_command == "off" then 
					CV_DebugMode(false);
				else
					-- no args passed, display help
					CV_Visual_BoxText("Usage:")
					CV_Visual_BoxText("-- '" .. PLUGIN_COMMAND_SLASH .. " debug on' to enable debug mode.")
					CV_Visual_BoxText("-- '" .. PLUGIN_COMMAND_SLASH .. " debug off' to diable debug mode.")
				end
				valid_command = true
			end

			if not valid_command then 
				CV_Visual_BoxText("Unrecognized command try again.")
			else
				CompVoice.SaveData()
			end

		else

			-- we're going to default to showing the CompVoice help screen. 
			print(USAGE)

		end

		CV_Visual_BoxFooter()

	end

-- END parser
-- ----------
-- BEGIN plugin initialization

	-- we separate this out so we can call it at plugin restart, or at login time. one uses an event the other checks to see if we're already in game.
	function CompVoice.BootPlugin()
		CV_SendToDebug("BootPlugin called")

		-- we only ever want to do this once at login, not at every time we dock.
		if not CompVoice.loggedin then
			CompVoice.LoadData() -- load or seed our config data

			for n = 0, 100 do
				if not LITE or (LITE and ((n==5) or (n==25) or (n==50) or (n==75))) then 
					if gksound.GKLoadSound{soundname = "f_"..tostring(n), filename = "plugins/CompVoice/audio/"..tostring(n)..".ogg", preload=true, vocal=true} then
						CompVoice.sound_objects["f_"..tostring(n)] = gksound.GKCreateSound("f_"..tostring(n))
					end
				end
			end

			if gksound.GKLoadSound{soundname = "armor_at", filename = "plugins/CompVoice/audio/armor_at.ogg", preload=true, vocal=true} then
				CompVoice.sound_objects["armor_at"] = gksound.GKCreateSound("armor_at")
			end
			
			if gksound.GKLoadSound{soundname = "percent", filename = "plugins/CompVoice/audio/percent.ogg", preload=true, vocal=true} then
				CompVoice.sound_objects["percent"] = gksound.GKCreateSound("percent")
			end

			if gksound.GKLoadSound{soundname = "shields_at", filename = "plugins/CompVoice/audio/shields_at.ogg", preload=true, vocal=true} then
				CompVoice.sound_objects["shields_at"] = gksound.GKCreateSound("shields_at")
			end
			
			if gksound.GKLoadSound{soundname = "ship_repaired", filename = "plugins/CompVoice/audio/ship_repaired.ogg", preload=true, vocal=true} then
				CompVoice.sound_objects["ship_repaired"] = gksound.GKCreateSound("ship_repaired")
			end

			if not LITE then

				if gksound.GKLoadSound{soundname = "docked", filename = "plugins/CompVoice/audio/docked.ogg", preload=true, vocal=true} then
					CompVoice.sound_objects["docked"] = gksound.GKCreateSound("docked")
				end
			
				if gksound.GKLoadSound{soundname = "shields_down", filename = "plugins/CompVoice/audio/shields_down.ogg", preload=true, vocal=true} then
					CompVoice.sound_objects["shields_down"] = gksound.GKCreateSound("shields_down")
				end

				if gksound.GKLoadSound{soundname = "shields_up", filename = "plugins/CompVoice/audio/shields_up.ogg", preload=true, vocal=true} then
					CompVoice.sound_objects["shields_up"] = gksound.GKCreateSound("shields_up")
				end

				if gksound.GKLoadSound{soundname = "you_have_died", filename = "plugins/CompVoice/audio/you_have_died.ogg", preload=true, vocal=true} then
					CompVoice.sound_objects["you_have_died"] = gksound.GKCreateSound("you_have_died")
				end

			end

			CV_DebugMode(CompVoice.Config.debug_mode) -- set our debug mode based on setting stored in saved config

			print("\127FFFFFF" .. PLUGIN_NAME .. " ready.\127o")
			CompVoice.loggedin = GetPlayerName()
		end

	end

-- -----

	-- Boot the plugin, and populate some key data that can't be populated without a player being logged in
	-- this is called by the registered event from initilization
	local function CV_PlayerLoggedIn(event, data)
		CV_SendToDebug("CV_PlayerLoggedIn called")
		if CompVoice.loggedin then -- have we booted the plugin yet?
			-- we've already booted, put stuff to do here.
		else
			-- we've never booted, put stuff to do here.
			CompVoice.BootPlugin()
		end
	end
	
	local function CV_UnloadInterfaceCalled(...)
		-- ReloadInterface called, so let's remove our sound objects for a clean reload afterward
			print("ReloadInterface() called: Unloading sound files...")
			for n = 0, 100 do
				if not LITE or (LITE and ((n==5) or (n==25) or (n==50) or (n==75))) then 
					CompVoice.sound_objects["f_"..tostring(n)] = nil
				end
			end
			CompVoice.sound_objects["armor_at"] = nil
			CompVoice.sound_objects["percent"] = nil
			CompVoice.sound_objects["shields_at"] = nil
			CompVoice.sound_objects["ship_repaired"] = nil
			if not LITE then
				CompVoice.sound_objects["docked"] = nil
				CompVoice.sound_objects["shields_down"] = nil
				CompVoice.sound_objects["shields_up"] = nil
				CompVoice.sound_objects["you_have_died"] = nil
			end
			print("Ready for reload.")
	end

-- -----

	-- this is all executed at plugin load time (usually game start, but also at ReloadInterface())
	-- locals rule, globals SUCK! Anti-global is the way to go!
	local init; do

		if LITE then 
			CompVoice.version = CompVoice.version .. ".L"
			LOGO = LOGO .. " -   L   I   T   E   - "
			PLUGIN_NAME = PLUGIN_NAME .. " Lite"
		end

		-- A little something for the errorlog at game load time
		console_print(LOGO) -- bragalicious

		-- Plugin user command(s) registrations
		RegisterUserCommand("cv", CompVoice.SlashCommandParser)

		-- Plugin EVENTS registrations
		RegisterEvent(CV_PlayerLoggedIn, "PLAYER_ENTERED_GAME")
		RegisterEvent(CV_UnloadInterfaceCalled, "UNLOAD_INTERFACE")

		-- Ship damanged
		RegisterEvent(CV_ShipGotDamaged, "PLAYER_GOT_HIT")
		
		-- Probably need to hook every way a ship could ever be updated...
		RegisterEvent(CV_ShipGotDamaged, "SHIP_UPDATED")
		RegisterEvent(CV_ShipGotDamaged, "SHIP_SPAWNED")
		
		-- we've changed ships
		RegisterEvent(CV_ShipChanged, "SHIP_CHANGED")

		-- check to see if we're logged in (ReloadInterface() used) or trap log in event
		-- here's a protip: GetPlayerName will not return a player name unless the plugin was loaded by a ReloadInterface(). 
		if GetPlayerName() then
			-- we got a player name, so this was a ReloadInterface()! Let's go ahead and launch our plugin
			print("\127FFFFFF" .. PLUGIN_NAME .. " INIT\127A0A0A0: Plugin loaded; User command '" .. PLUGIN_COMMAND_SLASH .. "' registered; Event hooks registered...\127o")
			CompVoice.BootPlugin()
		else
			-- we didn't get a player name, so we were loaded at game start, before logging in. let's wait for a login event to be triggered.
			print("\127FFFFFF" .. PLUGIN_NAME .. " INIT\127A0A0A0: Plugin pre-loaded; User command '" .. PLUGIN_COMMAND_SLASH .. "' registered; Event hooks registered... waiting for login.\127o")
		end

	end -- init

-- END plugin initialization
-- ----------
-- END PLUGIN